/*
Aero Color changing without restarting DWM (Desktop Window Manager).
Programed by : Tejashwi Kalp Taru
Dated : 06-Apr-2013.
Language : Windows API+C.
Environment : Windows 7 32bit + Visual Studio IDE.
Background :
	After analyzing dwmapi.dll in IDA I found two function
	A. DwmSetColorizationParameters
	B. DwmGetColorizationParameters
	These functions are used to get and set the DWM Color related values.
	As there functions are undocumented thus this functions can be removed by
	Microsoft at any time. Use in your app with your risk.
	Using Microsoft public symbol server I found the index of these functions.
	I used that index in GetProcAddress() function to load it's address.


You may use all or any part of the following code as long as you agree
to the Creative Commons Attribution 2.0 UK: England & Wales license.
http://creativecommons.org/licenses/by/2.0/uk/
*/

#include <windows.h>
#include <stdlib.h>
#include <time.h>
#include "resource.h"

HMODULE hDwmDLL;
BOOL enabled = FALSE;
time_t t;
int R=0,G=0,B=0; // Colors
#define ID_TIMER 1 // Timer ID
typedef struct COLORIZATIONPARAMS
{
	COLORREF         clrColor;		   //ColorizationColor
	COLORREF         clrAftGlow;	   //ColorizationAfterglow
	UINT             nIntensity;	   //ColorizationColorBalance -> 0-100
	UINT             clrAftGlowBal;    //ColorizationAfterglowBalance
	UINT			 clrBlurBal;       //ColorizationBlurBalance
	UINT			 clrGlassReflInt;  //ColorizationGlassReflectionIntensity
	BOOL             fOpaque;
}DWMColor;

DWMColor dwmcolor;
COLORREF Restore; // This will be used to restore back the colors to default

// Needed Function from DWM Library
HRESULT (WINAPI *DwmIsCompositionEnabled)(BOOL *pfEnabled);
HRESULT (WINAPI *DwmSetColorizationParameters) (COLORIZATIONPARAMS *colorparam,UINT unknown);
HRESULT (WINAPI *DwmGetColorizationParameters) (COLORIZATIONPARAMS *colorparam);

BOOL CALLBACK DlgProc(HWND hWnd,UINT message,WPARAM wParam,LPARAM lParam)
{
	switch(message)
	{
		case WM_INITDIALOG:
			hDwmDLL = LoadLibrary("dwmapi.dll"); // Loads the DWM DLL
			if(!hDwmDLL)
			{
				MessageBox(hWnd,"Unable to load DWM Library!","Error",MB_ICONERROR);
				EndDialog(hWnd,0); // Close
			}
			else
			{
				// Everything is fine upto here, we can get function address
				*(FARPROC *)&DwmIsCompositionEnabled = GetProcAddress(hDwmDLL,"DwmIsCompositionEnabled");
				//Below two functions are undocumented thus load from there index
				*(FARPROC *)&DwmGetColorizationParameters = GetProcAddress(hDwmDLL,(LPCSTR)127);
				*(FARPROC *)&DwmSetColorizationParameters = GetProcAddress(hDwmDLL,(LPCSTR)131);
			}
			DwmIsCompositionEnabled(&enabled);//Need to call DwmIsCompositionEnabled before calling the DwmSetColorizationParameters or it will fail.
			if(!enabled) // If glass composition is not enabled then close up
			{
				MessageBox(hWnd,"Composition is not enabled\nUnable to work!","Error",MB_ICONERROR);
				FreeLibrary(hDwmDLL);
				EndDialog(hWnd,0);
			}
			if(!DwmGetColorizationParameters && !DwmSetColorizationParameters) // The undocumented functions are not there,sorry
			{
				MessageBox(hWnd,"Unable to find necessary functions in library","Error",MB_ICONERROR);
				FreeLibrary(hDwmDLL);
				EndDialog(hWnd,0);
			}
			else
			{
				DwmGetColorizationParameters(&dwmcolor); // Get current values
				Restore = dwmcolor.clrColor; //Backup current color
				srand((unsigned) time(&t)); // seed for random number
			}
			return TRUE;
		case WM_TIMER:
			{
				R = (rand() % 256);
				G = (rand() % 256);
				B = (rand() % 256);
				dwmcolor.clrColor = RGB(R,G,B);
				DwmSetColorizationParameters(&dwmcolor,0);
			}
			return TRUE;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
			case IDC_BUTTON1: //Stop
				KillTimer(hWnd,ID_TIMER);
				dwmcolor.clrColor = Restore;
				DwmSetColorizationParameters(&dwmcolor,0); // Restore color back
				break;
			case IDC_BUTTON2: //Start
				SetTimer(hWnd,ID_TIMER,300,NULL);
				break;
			}
			return TRUE;
		case WM_CLOSE:
			KillTimer(hWnd,ID_TIMER);
			dwmcolor.clrColor = Restore;
			DwmSetColorizationParameters(&dwmcolor,0); // Restore color back
			FreeLibrary(hDwmDLL); // Free the loaded DWM Library
			EndDialog(hWnd,0);
			return TRUE;
	}
	return FALSE;
}

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nShowCmd)
{
	DialogBoxParam(hInstance,MAKEINTRESOURCE(IDD_DIALOG1),NULL,DLGPROC(DlgProc),0);
}